/*********************************************************************/
/* file: utilitys.cc - some utility-functions                        */
/*                             HUT TINTIN                            */
/*                coded by Ossi Tulonen from TinTin III              */
/*                TinTin III coded by peter unold 1992               */
/*********************************************************************/
/*
 * TinTin III code Copyright (c) 1992 Peter Unold
 * All changed code from TinTin III Copyright (c) 1992 1993 1994 Ossi tulonen
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
 /*
 *
 * HISTORY
 *
 * $Log: utility.cc,v $
// Revision 1.15  1994/11/26  06:20:40  oat
// v1.1
//
// Revision 1.14  1994/11/08  17:06:03  oat
// ht v1.0
//
// Revision 1.13  1994/10/07  14:49:52  oat
// added copyright text.
//
// Revision 1.12  1994/10/04  14:01:22  oat
// ht v 1.0 beta
//
// Revision 1.11  1994/09/28  05:20:39  oat
// ht V0.9_27
//
// Revision 1.10  1994/09/17  12:56:11  oat
// v 0.9_25
//
// Revision 1.9  1994/09/14  23:16:17  oat
// *** empty log message ***
//
// Revision 1.8  1994/08/30  13:55:40  oat
// *** empty log message ***
//
// Revision 1.7  1994/08/17  02:33:55  oat
// *** empty log message ***
//
// Revision 1.6  1994/08/12  08:53:18  oat
// old version before new screen history system ;)
//
// Revision 1.5  1994/08/12  07:52:12  oat
// str_ncpy() now knows ^G = '\a' = bell ascii char
//
// Revision 1.4  1994/07/31  00:02:42  oat
// now multi window screen should work.
// (so lets see how many bug fixes version come ;)
//
// Revision 1.3  1994/07/28  01:30:38  oat
// chars marked signed or unsigned
//
// Revision 1.2  1994/07/27  14:04:15  oat
// Bugs fixes.
//
// Revision 1.1  1994/07/26  17:18:56  oat
// Initial revision
//
 * Revision 1.2  1994/07/26  16:22:36  oat
 * lots of changes....
 *
 * Revision 1.1  1994/04/10  23:56:23  oat
 * Initial revision
 *
 * Revision 3.39  1994/03/11  19:49:43  oat
 * removed config.h
 *
 * Revision 3.38  1994/01/21  16:38:01  oat
 * changed to fit time data in last line.
 *
 * Revision 3.37  1994/01/20  08:05:15  oat
 * made str_ncpy() strip esc codes except ESC[?;?m codes (= colors, u
 * underlined, italised, ...)
 *
 *
 */

#include <assert.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#ifndef NSTDLIBS
#include <stdlib.h>
#endif

#include "structs.h"
#include "utility.h"

extern struct master_table_data master_table;
extern unsigned char text_start, text_end;

int is_abrev(const unsigned char *s1, const unsigned char *s2)
{
  while(*s1 && *s2)
    if(*s1++ != *s2++)
      return(FALSE);

  if(*s1 || (!*s1 && !*s2))
    return(TRUE);

  return(FALSE);
}

#ifdef NSTRDUP
// *******************************
//  strdup - duplicates a string
//  return: address of duplicate 
// *******************************
char *strdup(const char *s)
{
  char *dup;
  
  if((dup = (char *)malloc(strlen(s)+1)) == NULL)
    syserr("Not enought memory for strdup.");
  strcpy(dup, s);
  return dup;
}
#endif

/*************************************************/
/* print system call error message and terminate */
/*************************************************/
void syserr(const unsigned char *msg)
{
  extern int errno, sys_nerr;
  extern char *sys_errlist[];

  fprintf(stderr,"ERROR: %s (%d",msg, errno);
  if(errno>0 && errno<sys_nerr)
    fprintf(stderr,": %s)\n",sys_errlist[errno]);
  else
    fprintf(stderr,")\n");
  exit(1);
}

int str_cat(unsigned char *s, int len, int maxlen, const unsigned char *append)
{
  int i;
  
  assert(len <= maxlen);
  
  for ( i = 0; len + i < maxlen - 1 && *(append + i) != '\0'; i++)
    *(s + len + i) = *(append + i);
  
  *(s + len + i) = '\0';
  
  if(*(append + i) != '\0') {
    if(master_table.main_session->logfile)
      fprintf(master_table.main_session->logfile,
	      "### BUG: too long append(%d/%d) string in str_cat(max %d)\n%s",
	      i, (int)strlen((const char *)append),
	      maxlen,
	      append);
    else
      fprintf(stderr,
	      "### BUG: too long append(%d/%d) string in str_cat(max %d)\n%s",
	      i, (int)strlen((const char *)append),
	      maxlen,
	      append);
  }

  return(len + i);
}

int add_str(uchar *result, int &len, int maxlen, int &inside, const uchar *append,
	    int remspace, session *ses)
{
  int ret_value = FALSE;
  const int olen = len;
  
  if(len > 0 && len < maxlen - 1)
    *(result + len++) = ' ';
  
  if(remspace || len > 0) {
    while(isspace(*append))
      append++;
  }
  
  if(text_start != text_end) {
    while(*append && len < maxlen - 1) {
      if(*append == '\\') {
	result[len++] = *append++;
	if(*append) {
	  result[len++] = *append++;
	} else {
	  ret_value = TRUE;
	}
      } else if(*append == text_start) {
	inside++;
	result[len++] = *append++;
      } else if(*append == text_end) {
	if(inside > 0)
	  inside--;
	else {
	  ses->print((const uchar *)"\n\r#ERROR - Unmatched parenthesis error!\n\r");
	}
	result[len++] = *append++;
      } else {
	result[len++] = *append++;
      }
    }
    if(len > 0 && *(result + len - 1) == '\n')
      len--;

    if(remspace) {
      while(len > 0 && isspace(*(result + len - 1)))
	len--;
    }
    result[len] = '\0';
    
    if(*append) {
      ses->print((const uchar *)"\n\r###Line too long, trungated: ");
      ses->print(result + olen);
      ses->print((const uchar *)"\n\r");
    }
  } else {
    len = str_cat(result, len, maxlen, append);
    if(len > 0 && *(result + len - 1) == '\n')
      len--;
    
    if(remspace) {
      while(len > 0 && isspace(*(result + len - 1)))
	len--;
    }
    result[len] = '\0';
    if(len > 0 && *(result + len - 1) == '\\')
      ret_value = TRUE;
  }
  
  if(ret_value || inside > 0) {
    ret_value = TRUE;
    
    if(*(result + len - 1) == '\\')
      len--;
    
    while(len > 0 && isspace(*(result + len - 1)))
      len--;
  }
  *(result + len) = '\0';
  
  return(ret_value);
}

int str_len(const unsigned char *str)
{
  int len;

  len = 0;
  while(*str) {
    if(*str == ESC) {
      str++;
      if(*str == '[') {
        str++;
        while(isdigit(*str))
          str++;
        while(*str == ';') {
          str++;
          while(isdigit(*str))
            str++;
        }
	if(*str)
	  str++;
      }
    } else {
      switch(*str) {
      case '\b':                       // backspace
      case 127:                        // delete
	if(len > 0) {
	  len--;
	  str++;
	} else
	  str++;
	break;
      case '\t':                       // tab
	str++;
	len++;
	if(len % 8)
	  len += 8 - (len % 8);

	break;
      default:
	if(*str >= ' ') {
	  len++;
	  str++;
	} else {
	  str++;
	}
      }
    }
  }
  return(len);
}

int strip_ctrl_codes(unsigned char *s, const unsigned char *str)
{
  const unsigned char *org;
  
  org = s;
  while(*str) {
    if(*str == ESC) {
      str++;
      if(*str == '[') {
        str++;
        while(isdigit(*str))
	  str++;
	
        while(*str == ';') {
          str++;
          while(isdigit(*str))
            str++;
        }
	if(*str)
	  str++;
      } else {
	if(*str)
	  str++;
      }
    } else {
      if(*str >= ' ' || *str == '\n')
	*s++ = *str;
      str++;
    }
  }
  *s = '\0';
  
  return(s - org);
}

int is_text(const unsigned char *text, const unsigned char *str)
{
  while(*text && *str) {
    if(*str == '?') {
      text++;
      str++;
    } else if(*str == '\\') {
      str++;
      if(*str && *text++ != *str++)
	return(FALSE);
    } else {
      if(*text++ != *str++)
	return(FALSE);
    }
  }
  if(*text || (!*text && !*str))
    return(TRUE);
  
  return(FALSE);
}

int mystrcmp(const unsigned char *text, const unsigned char *str)
{
  unsigned char *ptmp2, tmp2[BUFFER_SIZE];
  int len, num;

  if(*str != '*') {
    ptmp2 = tmp2;
    len = 0;
    while(*str && *str != '*') {
      if(*str == '\\')
	*ptmp2++ = *str++;
      
      if(*str) {
	*ptmp2++ = *str++;
	len++;
      }
    }
    *ptmp2 = '\0';
    
    if(is_text(text, tmp2)) {
      text += len;
    } else
      return FALSE;
  }
  
  while(*text && *str) {
    num = 0;
    while(*str == '*' || *str == '?') {
      if(*str == '?')
	num++;
      str++;
    }
    while(*text && num > 0) {
      text++;
      num--;
    }
    if(num > 0)              // not enought letter for '?'s
      return(FALSE);
    
    ptmp2 = tmp2;
    len = 0;
    while(*str && *str != '*') {
      if(*str == '\\') {
	*ptmp2++ = *str++;
	
	if(*str) {
	  *ptmp2++ = *str++;
	  len++;
	}
      } else {
	*ptmp2++ = *str++;
	len++;
      }
    }
    *ptmp2 = '\0';
    
    if(!*tmp2)            // '*' was last char
      return(TRUE);

    while(*text && !is_text(text, tmp2))
      text++;
    
    if(*text) {
      text += ptmp2 - tmp2;
    } else
      return FALSE;
  }
  
  if(*text)
    return FALSE;
  else if(!*str || (*str == '*' && !*(str + 1)))
    return TRUE;
  
  return FALSE;
}

#ifndef STRICMP

#define LOWER(c) (((c)>='A'  && (c) <= 'Z') ? ((c)+('a'-'A')) : (c))

/* returns: 0 if equal, 1 if arg1 > arg2, -1 if arg1 < arg2  */
/* scan 'till found different or end of both                 */
int stricmp(const unsigned char *arg1, const unsigned char *arg2)
{
  int chk, i;
  
  for (i = 0; *(arg1 + i) || *(arg2 + i); i++)
    if ((chk = LOWER(*(arg1 + i)) - LOWER(*(arg2 + i))))
      if (chk < 0)
	return (-1);
      else 
	return (1);
  return(0);
}
#endif

hutchar::hutchar()
{
  text = 0;
  len = 0;
}

hutchar::~hutchar()
{
  if(text)
    free(text);
  text = 0;
  len = 0;
}

int ownisalnum(const unsigned char letter)
{
  if(isalpha(letter) ||
     isdigit(letter) ||
     letter == (unsigned char)'\345' ||         // Finnish special letters
     letter == (unsigned char)'\344' ||         // from latin-1
     letter == (unsigned char)'\366' ||         // ''
     letter == (unsigned char)'\305' ||
     letter == (unsigned char)'\304' ||
     letter == (unsigned char)'\326')
    return(TRUE);
  return(FALSE);
}
